@@ -119,10 +119,10 @@ class FD(DelayComponent):
         d_delay_d_FD = np.polyval(FD_coeff, log_freq)
         return d_delay_d_FD * u.second / FD_par.units
 
-    def print_par(self):
+    def print_par(self, format="pint"):
         result = ""
         FD_mapping = self.get_prefix_mapping_component("FD")
         for FD in FD_mapping.values():
             FD_par = getattr(self, FD)
-            result += FD_par.as_parfile_line()
+            result += FD_par.as_parfile_line(format=format)
         return result
