@@ -49,6 +49,11 @@ from pint.utils import split_prefixed_name
 log = logging.getLogger(__name__)
 
 
+# potential parfile formats
+# in one place for consistency
+_parfile_formats = ["pint", "tempo", "tempo2"]
+
+
 def _identity_function(x):
     """A function to just return the input argument
 
@@ -175,6 +180,9 @@ class Parameter:
     ):
 
         self.name = name  # name of the parameter
+        # The input parameter from parfile, which can be an alias of the parameter
+        # TODO give a better name and make it easy to access.
+        self._parfile_name = name
         self.units = units  # Default unit
         self.quantity = value  # The value of parameter, internal storage
         self.prior = prior
@@ -188,7 +196,6 @@ class Parameter:
         self.paramType = "Not specified"  # Type of parameter. Here is general type
         self.valueType = None
         self.special_arg = []
-
         self.use_alias = use_alias
 
     @property
@@ -359,6 +366,10 @@ class Parameter:
         if val != 0:
             raise NotImplementedError()
 
+    @property
+    def repeatable(self):
+        return False
+
     @property
     def prior(self):
         """prior distribution for this parameter.
@@ -425,8 +436,30 @@ class Parameter:
             out += " (" + str(self.units) + ")"
         return out
 
-    def as_parfile_line(self):
-        """Return a parfile line giving the current state of the parameter."""
+    def as_parfile_line(self, format="pint"):
+        """Return a parfile line giving the current state of the parameter.
+
+        Parameters
+        ----------
+        format : str, optional
+             Parfile output format. PINT outputs in 'tempo', 'tempo2' and 'pint'
+             formats. The defaul format is `pint`.
+
+        Returns
+        -------
+        str
+
+        Notes
+        -----
+        Format differences between tempo, tempo2, and pint at [1]_
+
+        .. [1] https://github.com/nanograv/PINT/wiki/PINT-vs.-TEMPO%282%29-par-file-changes
+        """
+        assert format.lower() in _parfile_formats, (
+            "parfile format must be one of %s"
+            % ", ".join(['"%s"' % x for x in _parfile_formats])
+        )
+
         # Don't print unset parameters
         if self.quantity is None:
             return ""
@@ -434,7 +467,47 @@ class Parameter:
             name = self.name
         else:
             name = self.use_alias
+
+        # special cases for parameter names that change depending on format
+        if self.name == "CHI2" and not (format.lower() == "pint"):
+            # no CHI2 for TEMPO/TEMPO2
+            return ""
+        elif self.name == "SWM" and not (format.lower() == "pint"):
+            # no SWM for TEMPO/TEMPO2
+            return ""
+        elif self.name == "A1DOT" and not (format.lower() == "pint"):
+            # change to XDOT for TEMPO/TEMPO2
+            name = "XDOT"
+        elif self.name == "STIGMA" and not (format.lower() == "pint"):
+            # change to VARSIGMA for TEMPO/TEMPO2
+            name = "VARSIGMA"
+
+        # standard output formatting
         line = "%-15s %25s" % (name, self.str_quantity(self.quantity))
+        # special cases for parameter values that change depending on format
+        if self.name == "ECL" and format.lower() == "tempo2":
+            if self.value != "IERS2003":
+                warn(
+                    f"Changing ECL from '{self.value}' to 'IERS2003'; please refit for consistent results"
+                )
+                # change ECL value to IERS2003 for TEMPO2
+                line = "%-15s %25s" % (name, "IERS2003")
+        elif self.name == "NHARMS" and not (format.lower() == "pint"):
+            # convert NHARMS value to int
+            line = "%-15s %25d" % (name, self.value)
+        elif self.name == "KIN" and format.lower() == "tempo":
+            # convert from DT92 convention to IAU
+            line = "%-15s %25s" % (name, self.str_quantity(180 * u.deg - self.quantity))
+            warn(
+                f"Changing KIN from DT92 convention to IAU: this will not be readable by PINT"
+            )
+        elif self.name == "KOM" and format.lower() == "tempo":
+            # convert from DT92 convention to IAU
+            line = "%-15s %25s" % (name, self.str_quantity(90 * u.deg - self.quantity))
+            warn(
+                f"Changing KOM from DT92 convention to IAU: this will not be readable by PINT"
+            )
+
         if self.uncertainty is not None:
             line += " %d %s" % (
                 0 if self.frozen else 1,
@@ -442,6 +515,10 @@ class Parameter:
             )
         elif not self.frozen:
             line += " 1"
+
+        if self.name == "T2CMETHOD" and format.lower() == "tempo2":
+            # comment out T2CMETHOD for TEMPO2
+            line = "#" + line
         return line + "\n"
 
     def from_parfile_line(self, line):
@@ -489,8 +566,8 @@ class Parameter:
                     str2longdouble(k[2])
                     ucty = k[2]
                 except ValueError:
-                    errmsg = "Unidentified string " + k[2] + " in"
-                    errmsg += " parfile line " + k
+                    errmsg = f"Unidentified string '{k[2]}' in"
+                    errmsg += f" parfile line " + " ".join(k)
                     raise ValueError(errmsg)
 
             if len(k) >= 4:
@@ -1238,7 +1315,7 @@ class prefixParameter:
         scale_factor=None,
         scale_threshold=None,
         time_scale="utc",
-        **kwargs,  # FIXME: why is this here?
+        **kwargs,
     ):
         # Split prefixed name, if the name is not in the prefixed format, error
         # will be raised
@@ -1300,6 +1377,10 @@ class prefixParameter:
         self.is_prefix = True
         self.time_scale = time_scale
 
+    @property
+    def repeatable(self):
+        return self.param_comp.repeatable
+
     @property
     def units(self):
         return self.param_comp.units
@@ -1356,6 +1437,14 @@ class prefixParameter:
     def aliases(self, a):
         self.param_comp.aliases = a
 
+    @property
+    def use_alias(self):
+        return self.param_comp.use_alias
+
+    @use_alias.setter
+    def use_alias(self, a):
+        self.param_comp.use_alias = a
+
     @property
     def continuous(self):
         return self.param_comp.continuous
@@ -1402,8 +1491,8 @@ class prefixParameter:
     def name_matches(self, name):
         return self.param_comp.name_matches(name)
 
-    def as_parfile_line(self):
-        return self.param_comp.as_parfile_line()
+    def as_parfile_line(self, format="pint"):
+        return self.param_comp.as_parfile_line(format=format)
 
     def help_line(self):
         return self.param_comp.help_line()
@@ -1568,7 +1657,7 @@ class maskParameter(floatParameter):
             uncertainty=uncertainty,
             frozen=frozen,
             continuous=continuous,
-            aliases=idx_aliases,
+            aliases=idx_aliases + aliases,
             long_double=long_double,
         )
 
@@ -1577,6 +1666,7 @@ class maskParameter(floatParameter):
         if index == 1:
             self.aliases.append(name)
         self.is_prefix = True
+        self._parfile_name = self.origin_name
 
     def __repr__(self):
         out = self.__class__.__name__ + "(" + self.name
@@ -1598,6 +1688,10 @@ class maskParameter(floatParameter):
 
         return out
 
+    @property
+    def repeatable(self):
+        return True
+
     def name_matches(self, name):
         if super().name_matches(name):
             return True
@@ -1691,13 +1785,27 @@ class maskParameter(floatParameter):
             self.uncertainty = self._set_uncertainty(ucty)
         return True
 
-    def as_parfile_line(self):
+    def as_parfile_line(self, format="pint"):
+        assert format.lower() in _parfile_formats, (
+            "parfile format must be one of %s"
+            % ", ".join(['"%s"' % x for x in _parfile_formats])
+        )
+
         if self.quantity is None:
             return ""
         if self.use_alias is None:
             name = self.origin_name
         else:
             name = self.use_alias
+
+        # special cases for parameter names that change depending on format
+        if name == "EFAC" and not (format.lower() == "pint"):
+            # change to T2EFAC for TEMPO/TEMPO2
+            name = "T2EFAC"
+        elif name == "EQUAD" and not (format.lower() == "pint"):
+            # change to T2EQUAD for TEMPO/TEMPO2
+            name = "T2EQUAD"
+
         line = "%-15s %s " % (name, self.key)
         for kv in self.key_value:
             if isinstance(kv, time.Time):
@@ -1752,11 +1860,7 @@ class maskParameter(floatParameter):
         array
             An array of TOA indices selected by the mask.
         """
-        column_match = {
-            "mjd": "mjd_float",
-            "freq": "freq",
-            "tel": "obs",
-        }
+        column_match = {"mjd": "mjd_float", "freq": "freq", "tel": "obs"}
         if len(self.key_value) == 1:
             if not hasattr(self, "toa_selector"):
                 self.toa_selector = TOASelect(is_range=False, use_hash=True)
@@ -1795,6 +1899,30 @@ class maskParameter(floatParameter):
         select_idx = self.toa_selector.get_select_index(condition, col)
         return select_idx[self.name]
 
+    def compare_key_value(self, other_param):
+        """Compare if the key and value are the same with the other parameter.
+
+        Parameter
+        ---------
+        other_param: maskParameter
+            The parameter to compare.
+        Return
+        ------
+        bool:
+            If the key and value are the same, return True, otherwise False.
+        Raises
+        ------
+        ValueError:
+            If the parameter to compare does not have 'key' or 'key_value'.
+        """
+        if not (hasattr(other_param, "key") or hasattr(other_param, "key_value")):
+            raise ValueError("Parameter to compare does not have `key` or `key_value`.")
+        if self.key != other_param.key:
+            return False
+        if self.key_value != other_param.key_value:
+            return False
+        return True
+
 
 class pairParameter(floatParameter):
     """Parameter type for parameters that need two input floats.
@@ -1893,7 +2021,7 @@ class pairParameter(floatParameter):
 
         return True
 
-    def as_parfile_line(self):
+    def as_parfile_line(self, format="pint"):
         quantity = self.quantity
         if self.quantity is None:
             return ""
