@@ -118,8 +118,8 @@ class SolarWindDispersion(Dispersion):
         deriv[bfreq < 1.0 * u.MHz] = 0.0
         return deriv
 
-    def print_par(self):
+    def print_par(self, format="pint"):
         result = ""
-        result += getattr(self, "NE_SW").as_parfile_line()
-        result += getattr(self, "SWM").as_parfile_line()
+        result += getattr(self, "NE_SW").as_parfile_line(format=format)
+        result += getattr(self, "SWM").as_parfile_line(format=format)
         return result
