@@ -1,14 +1,16 @@
 """Kopeikin corrected DD model."""
-from __future__ import absolute_import, division, print_function
+import logging
 
 import astropy.constants as c
 import astropy.units as u
 import numpy as np
-from astropy import log
+
 from pint import GMsun, Tsun, ls
 
 from .DD_model import DDmodel
 
+log = logging.getLogger(__name__)
+
 
 class DDKmodel(DDmodel):
     """DDK model, a Kopeikin method corrected DD model.
