@@ -1,10 +1,9 @@
-# This file is a prototype of independent psr binary model class
-from __future__ import absolute_import, division, print_function
+"""Parent class for internal binary models."""
+import logging
 
 import astropy.constants as c
 import astropy.units as u
 import numpy as np
-from astropy import log
 
 try:
     from erfa import DAYSEC as SECS_PER_DAY
@@ -14,10 +13,12 @@ except ImportError:
 from pint import Tsun, ls
 from pint.models.stand_alone_psr_binaries.binary_orbits import OrbitPB
 
+log = logging.getLogger(__name__)
+
 SECS_PER_JUL_YEAR = SECS_PER_DAY * 365.25
 
 
-class PSR_BINARY(object):
+class PSR_BINARY:
     """A base (generic) object for psr binary models.
 
     In this class, a set of generally used binary paramters and several commonly used
