@@ -379,10 +379,10 @@ def tempo_polyco_table_writer(polycoTable, filename="polyco.dat"):
     f.close()
 
 
-class Polycos(object):
-    """
-    A class for polycos model. Polyco is a fast phase calculator. It fits a set
-    of data using polynomials.
+class Polycos:
+    """A class for polycos model.
+
+    Polyco is a fast phase calculator. It fits a set of data using polynomials.
     """
 
     def __init__(self):
@@ -567,7 +567,7 @@ class Polycos(object):
                     ]
                 )
 
-                refPhase = model.phase(toaMid)
+                refPhase = model.phase(toaMid, abs_phase=True)
 
                 # Create node toas(Time sample using TOA class)
                 toaList = [
@@ -581,7 +581,7 @@ class Polycos(object):
 
                 toas = toa.get_TOAs_list(toaList)
 
-                ph = model.phase(toas)
+                ph = model.phase(toas, abs_phase=True)
                 dt = (nodes - tmid) * MIN_PER_DAY
                 rdcPhase = ph - refPhase
                 rdcPhase = rdcPhase.int - (dt * model.F0.value * 60.0) + rdcPhase.frac
