@@ -6,19 +6,20 @@ objects, as ``fitter.residual``. Variants exist for arrival-time-only data
 (:class:`pint.residuals.Residuals`) and for arrival times that come paired with
 dispersion measures (:class:`pint.residuals.WidebandTOAResiduals`).
 """
-from __future__ import absolute_import, division, print_function
-
 import collections
+import copy
+import logging
 import warnings
 
 import astropy.units as u
 import numpy as np
-from astropy import log
 from scipy.linalg import LinAlgError
 
 from pint.models.dispersion_model import Dispersion
 from pint.phase import Phase
-from pint.utils import weighted_mean
+from pint.utils import weighted_mean, taylor_horner_deriv
+
+log = logging.getLogger(__name__)
 
 __all__ = [
     "Residuals",
@@ -69,8 +70,10 @@ class Residuals:
         Controls how pulse numbers are assigned. ``"nearest"`` assigns
         each TOA to the nearest integer pulse. ``"use_pulse_numbers"`` uses the
         ``pulse_number`` column of the TOAs table to assign pulse numbers. If the
-        default, None, is passed, use the pulse numbers if and only if the model has
-        parameter TRACK == "-2".
+        default, None, is passed, use the pulse numbers if the model has the
+        parameter TRACK == "-2" and not if it has TRACK == "0". If neither of the
+        above is set, use pulse numbers if there are pulse numbers present and not
+        if there aren't.
     """
 
     def __new__(
@@ -114,6 +117,16 @@ class Residuals:
         if track_mode is None:
             if getattr(self.model, "TRACK").value == "-2":
                 self.track_mode = "use_pulse_numbers"
+            elif getattr(self.model, "TRACK").value == "0":
+                self.track_mode = "nearest"
+            elif "pulse_number" in self.toas.table.columns:
+                if np.any(np.isnan(toas.table["pulse_number"])):
+                    log.warn(
+                        "Some TOAs are missing pulse numbers, they will not be used."
+                    )
+                    self.track_mode = "nearest"
+                else:
+                    self.track_mode = "use_pulse_numbers"
             else:
                 self.track_mode = "nearest"
         else:
@@ -129,6 +142,8 @@ class Residuals:
         # only relevant if there are correlated errors
         self._chi2 = None
         self.noise_resids = {}
+        # For residual debugging
+        self.debug_info = {}
         # We should be carefully for the other type of residuals
         self.unit = unit
         # A flag to indentify if this residual object is combined with residual
@@ -160,7 +175,6 @@ class Residuals:
         self.phase_resids = self.calc_phase_resids()
         self.time_resids = self.calc_time_resids()
         self._chi2 = None  # trigger chi2 recalculation when needed
-        self.dof = self.dof
 
     @property
     def chi2(self):
@@ -227,21 +241,24 @@ class Residuals:
         wmean, werr, wsdev = weighted_mean(self.time_resids, w, sdev=True)
         return wsdev.to(u.us)
 
-    def get_PSR_freq(self, modelF0=True):
+    def get_PSR_freq(self, calctype="modelF0"):
         """Return pulsar rotational frequency in Hz.
 
         Parameters
         ----------
-        modelF0 : bool, optional
-            If True, simply read the ``F0`` parameter from the model. If False,
-            query the model for the spin period at the time of each TOA.
+        calctype : {'modelF0', 'numerical', 'taylor'}
+            Type of calculation.  If `calctype` == "modelF0", then simply the ``F0``
+            parameter from the model.
+            If `calctype` == "numerical", then try a numerical derivative
+            If `calctype` == "taylor", evaluate the frequency with a Taylor series
 
         Returns
         -------
-        freq : Quantity
+        freq : astropy.units.Quantity
             Either the single ``F0`` in the model or the spin frequency at the moment of each TOA.
         """
-        if modelF0:
+        assert calctype.lower() in ["modelf0", "taylor", "numerical"]
+        if calctype.lower() == "modelf0":
             # TODO this function will be re-write and move to timing model soon.
             # The following is a temproary patch.
             if "Spindown" in self.model.components:
@@ -253,7 +270,23 @@ class Residuals:
                     "No pulsar spin parameter(e.g., 'F0'," " 'P0') found."
                 )
             return F0.to(u.Hz)
-        else:
+        elif calctype.lower() == "taylor":
+            # see Spindown.spindown_phase
+            dt = self.model.get_dt(self.toas, 0)
+            # if the model is defined through F0, F1, ...
+            if "F0" in self.model.params:
+                fterms = [0.0 * u.dimensionless_unscaled] + self.model.get_spin_terms()
+
+            # otherwise assume P0, PDOT
+            else:
+                F0 = 1.0 / self.model.P0.quantity
+                if "PDOT" in self.model.params:
+                    F1 = -self.model.PDOT.quantity / self.model.P0.quantity ** 2
+                else:
+                    F1 = 0 * u.Hz / u.s
+                fterms = [0.0 * u.dimensionless_unscaled, F0, F1]
+            return taylor_horner_deriv(dt, fterms, deriv_order=1).to(u.Hz)
+        elif calctype.lower() == "numerical":
             return self.model.d_phase_d_toa(self.toas)
 
     def calc_phase_resids(self):
@@ -283,9 +316,17 @@ class Residuals:
             )
             # First assign each TOA to the correct relative pulse number, including
             # and delta_pulse_numbers (from PHASE lines or adding phase jumps in GUI)
-            residualphase = modelphase - Phase(pulse_num, np.zeros_like(pulse_num))
+            i = pulse_num.copy()
+            f = np.zeros_like(pulse_num)
+            c = np.isnan(pulse_num)
+            if np.any(c):
+                raise ValueError("Pulse numbers are missing on some TOAs")
+                i[c] = 0
+            residualphase = modelphase - Phase(i, f)
             # This converts from a Phase object to a np.float128
             full = residualphase.int + residualphase.frac
+            if np.any(c):
+                full[c] -= np.round(full[c])
         # If not tracking then do the usual nearest pulse number calculation
         elif self.track_mode == "nearest":
             # Compute model phase
@@ -310,19 +351,40 @@ class Residuals:
         else:
             # Errs for weighted sum.  Units don't matter since they will
             # cancel out in the weighted sum.
-            if np.any(self.toas.get_errors() == 0):
+            if np.any(self.get_data_error() == 0):
                 raise ValueError(
                     "Some TOA errors are zero - cannot calculate residuals"
                 )
-            w = 1.0 / (self.toas.get_errors().value ** 2)
+            w = 1.0 / (self.get_data_error().value ** 2)
             mean, err = weighted_mean(full, w)
         return full - mean
 
-    def calc_time_resids(self):
-        """Compute timing model residuals in time (seconds)."""
+    def calc_time_resids(self, calctype="taylor"):
+        """Compute timing model residuals in time (seconds).
+
+        Converts from phase residuals to time residuals using several possible ways
+        to calculate the frequency.
+
+        Parameters
+        ----------
+        calctype : {'taylor', 'modelF0', 'numerical'}
+            Type of calculation.  If `calctype` == "modelF0", then simply the ``F0``
+            parameter from the model.
+            If `calctype` == "numerical", then try a numerical derivative
+            If `calctype` == "taylor", evaluate the frequency with a Taylor series
+
+        Returns
+        -------
+        residuals : astropy.units.Quantity
+
+        See Also
+        --------
+        :meth:`pint.residuals.get_PSR_freq`
+        """
+        assert calctype.lower() in ["modelf0", "taylor", "numerical"]
         if self.phase_resids is None:
             self.phase_resids = self.calc_phase_resids()
-        return (self.phase_resids / self.get_PSR_freq()).to(u.s)
+        return (self.phase_resids / self.get_PSR_freq(calctype=calctype)).to(u.s)
 
     def calc_chi2(self, full_cov=False):
         """Return the weighted chi-squared for the model and toas.
@@ -344,12 +406,15 @@ class Residuals:
         correctly return infinity.
         """
         if self.model.has_correlated_errors:
+            log.debug("Using GLS fitter to compute residual chi2")
             # Use GLS but don't actually fit
             from pint.fitter import GLSFitter
 
-            f = GLSFitter(self.toas, self.model, residuals=self)
+            m = copy.deepcopy(self.model)
+            m.free_params = []
+            f = GLSFitter(self.toas, m, residuals=self)
             try:
-                return f.fit_toas(maxiter=0, full_cov=full_cov)
+                return f.fit_toas(maxiter=1, full_cov=full_cov)
             except LinAlgError as e:
                 log.warning(
                     "Degenerate conditions encountered when "
@@ -490,6 +555,8 @@ class WidebandDMResiduals(Residuals):
         self.dm_data, self.dm_error, self.relevant_toas = self.get_dm_data()
         self._chi2 = None
         self._is_combined = False
+        # For residual debugging
+        self.debug_info = {}
 
     @property
     def resids(self):
@@ -589,8 +656,8 @@ class WidebandDMResiduals(Residuals):
         valid_index: list
             The TOA with DM data index.
         """
-        dm_data, valid_data = self.toas.get_flag_value("pp_dm")
-        dm_error, valid_error = self.toas.get_flag_value("pp_dme")
+        dm_data, valid_data = self.toas.get_flag_value("pp_dm", as_type=float)
+        dm_error, valid_error = self.toas.get_flag_value("pp_dme", as_type=float)
         if valid_data == []:
             raise ValueError("Input TOA object does not have wideband DM values")
         # Check valid error, if an error is none, change it to zero
@@ -634,6 +701,8 @@ class CombinedResiduals:
         for res in residuals:
             res._is_combined = True
             self.residual_objs[res.residual_type] = res
+        # For residual debugging
+        self.debug_info = {}
 
     @property
     def model(self):
@@ -728,6 +797,7 @@ class WidebandTOAResiduals(CombinedResiduals):
             self.toas, self.model, residual_type="toa", **toa_resid_args
         )
         dm_resid = Residuals(self.toas, self.model, residual_type="dm", **dm_resid_args)
+        self._chi2 = None
 
         super().__init__([toa_resid, dm_resid])
 
@@ -741,6 +811,51 @@ class WidebandTOAResiduals(CombinedResiduals):
         """WidebandDMResiduals object containing the DM residuals."""
         return self.residual_objs["dm"]
 
+    @property
+    def chi2(self):
+        """Compute chi-squared as needed and cache the result."""
+        if self._chi2 is None:
+            self._chi2 = self.calc_chi2()
+        assert self._chi2 is not None
+        return self._chi2
+
+    def calc_chi2(self, full_cov=False):
+        """Return the weighted chi-squared for the model and toas.
+
+        If the errors on the TOAs are independent this is a straightforward
+        calculation, but if the noise model introduces correlated errors then
+        obtaining a meaningful chi-squared value requires a Cholesky
+        decomposition. This is carried out, here, by constructing a GLSFitter
+        and asking it to do the chi-squared computation but not a fit.
+
+        The return value here is available as self.chi2, which will not
+        redo the computation unless necessary.
+
+        The chi-squared value calculated here is suitable for use in downhill
+        minimization algorithms and Bayesian approaches.
+
+        Handling of problematic results - degenerate conditions explored by
+        a minimizer for example - may need to be checked to confirm that they
+        correctly return infinity.
+        """
+        log.debug("Using wideband GLS fitter to compute residual chi2")
+        # Use GLS but don't actually fit
+        from pint.fitter import WidebandTOAFitter
+
+        m = copy.deepcopy(self.model)
+        m.free_params = []
+        f = WidebandTOAFitter(
+            self.toas, m, additional_args=dict(toa=dict(track_mode=self.toa.track_mode))
+        )
+        try:
+            return f.fit_toas(maxiter=1, full_cov=full_cov)
+        except LinAlgError as e:
+            log.warning(
+                "Degenerate conditions encountered when "
+                "computing chi-squared: %s" % (e,)
+            )
+            return np.inf
+
     @property
     def model(self):
         """The model used to construct the residuals.
