@@ -1,7 +1,6 @@
 #!/usr/bin/env python -W ignore::FutureWarning -W ignore::UserWarning -W ignore::DeprecationWarning
-from __future__ import absolute_import, division, print_function
-
 import argparse
+import logging
 import os
 import sys
 
@@ -10,7 +9,6 @@ import astropy.units as u
 import matplotlib.pyplot as plt
 import numpy as np
 import scipy.optimize as op
-from astropy import log
 from astropy.coordinates import SkyCoord
 from scipy.stats import norm, uniform
 
@@ -28,6 +26,8 @@ from pint.models.priors import (
 )
 from pint.observatory.satellite_obs import get_satellite_observatory
 
+log = logging.getLogger(__name__)
+
 __all__ = ["read_gaussfitfile", "marginalize_over_phase", "main"]
 # log.setLevel('DEBUG')
 # np.seterr(all='raise')
@@ -413,6 +413,17 @@ class emcee_fitter(Fitter):
             plt.savefig(ftr.model.PSR.value + "_htest_v_wgtcut_unweighted.png")
         plt.close()
 
+    def plot_priors(self, chains, burnin, bins=100, scale=False):
+        plot_utils.plot_priors(
+            self.model,
+            chains,
+            self.maxpost_fitvals,
+            self.fitvals,
+            burnin=burnin,
+            bins=bins,
+            scale=scale,
+        )
+
 
 def main(argv=None):
 
@@ -552,12 +563,13 @@ def main(argv=None):
     target = tc if weightcol == "CALC" else None
 
     # TODO: make this properly handle long double
-    if not args.usepickle or (
-        not (
-            os.path.isfile(eventfile + ".pickle")
-            or os.path.isfile(eventfile + ".pickle.gz")
-        )
-    ):
+    ts = None
+    if args.usepickle:
+        try:
+            ts = toa.load_pickle(eventfile)
+        except IOError:
+            pass
+    if ts is None:
         # Read event file and return list of TOA objects
         tl = fermi.load_Fermi_TOAs(
             eventfile, weightcolumn=weightcol, targetcoord=target, minweight=minWeight
@@ -569,7 +581,7 @@ def main(argv=None):
             if (
                 tl[ii].mjd.value > minMJD
                 and tl[ii].mjd.value < maxMJD
-                and (weightcol is None or tl[ii].flags["weight"] > minWeight)
+                and (weightcol is None or float(tl[ii].flags["weight"]) > minWeight)
             )
         ]
         log.info("There are %d events we will use" % len(tl))
@@ -578,16 +590,11 @@ def main(argv=None):
         ts.filename = eventfile
         ts.compute_TDBs()
         ts.compute_posvels(ephem="DE421", planets=False)
-        ts.pickle()
-    else:  # read the events in as a pickle file
-        picklefile = toa._check_pickle(eventfile)
-        if not picklefile:
-            picklefile = eventfile
-        ts = toa.TOAs(picklefile)
+        toa.save_pickle(ts)
 
     if weightcol is not None:
         if weightcol == "CALC":
-            weights = np.asarray([x["weight"] for x in ts.table["flags"]])
+            weights = np.asarray([float(x["weight"]) for x in ts.table["flags"]])
             log.info(
                 "Original weights have min / max weights %.3f / %.3f"
                 % (weights.min(), weights.max())
@@ -599,8 +606,8 @@ def main(argv=None):
                 # make the highest weight = 1, but keep min weight the same
                 weights = wmn + ((weights - wmn) * (1.0 - wmn) / (wmx - wmn))
             for ii, x in enumerate(ts.table["flags"]):
-                x["weight"] = weights[ii]
-        weights = np.asarray([x["weight"] for x in ts.table["flags"]])
+                x["weight"] = str(weights[ii])
+        weights = np.asarray([float(x["weight"]) for x in ts.table["flags"]])
         log.info(
             "There are %d events, with min / max weights %.3f / %.3f"
             % (len(weights), weights.min(), weights.max())
@@ -787,6 +794,11 @@ def main(argv=None):
     except ImportError:
         pass
 
+    # Plot the scaled prior probability alongside the initial gaussian probability distribution and the histogrammed samples
+    ftr.plot_priors(chains, burnin, scale=True)
+    plt.savefig(ftr.model.PSR.value + "_priors.png")
+    plt.close()
+
     # Make a phaseogram with the 50th percentile values
     # ftr.set_params(dict(zip(ftr.fitkeys, np.percentile(samples, 50, axis=0))))
     # Make a phaseogram with the best MCMC result
@@ -828,6 +840,6 @@ def main(argv=None):
     f.write(ftr.model.as_parfile())
     f.close()
 
-    from six.moves import cPickle as pickle
+    import pickle
 
     pickle.dump(samples, open(ftr.model.PSR.value + "_samples.pickle", "wb"))
