@@ -1,23 +1,58 @@
-"""Miscellaneous potentially-helpful functions."""
-from __future__ import absolute_import, division, print_function
+"""Miscellaneous potentially-helpful functions.
 
+Warning
+-------
+Functions:
+
+- :func:`~pint.derived_quantities.a1sini`
+- :func:`~pint.derived_quantities.companion_mass`
+- :func:`~pint.derived_quantities.gamma`
+- :func:`~pint.derived_quantities.mass_funct`
+- :func:`~pint.derived_quantities.mass_funct2`
+- :func:`~pint.derived_quantities.omdot`
+- :func:`~pint.derived_quantities.omdot_to_mtot`
+- :func:`~pint.derived_quantities.p_to_f`
+- :func:`~pint.derived_quantities.pbdot`
+- :func:`~pint.derived_quantities.pferrs`
+- :func:`~pint.derived_quantities.pulsar_B`
+- :func:`~pint.derived_quantities.pulsar_B_lightcyl`
+- :func:`~pint.derived_quantities.pulsar_age`
+- :func:`~pint.derived_quantities.pulsar_edot`
+- :func:`~pint.derived_quantities.pulsar_mass`
+- :func:`~pint.derived_quantities.shklovskii_factor`
+
+have moved to :mod:`pint.derived_quantities`.
+
+- :func:`pint.simulation.calculate_random_models`
+
+has moved to :mod:`pint.simulation`.
+
+"""
+import configparser
+import datetime
+import getpass
+import logging
+import os
+import platform
 import re
+import textwrap
+from collections import OrderedDict
 from contextlib import contextmanager
 from copy import deepcopy
-import astropy.units as u
-from astropy import log
+from io import StringIO
+
 import astropy.constants as const
 import astropy.coordinates as coords
+import astropy.coordinates.angles as angles
+import astropy.units as u
 import numpy as np
-import six
 import scipy.optimize.zeros as zeros
 from scipy.special import fdtrc
 
-
-from io import StringIO
-
+import pint
 import pint.pulsar_ecliptic
 
+log = logging.getLogger(__name__)
 
 __all__ = [
     "PosVel",
@@ -33,32 +68,24 @@ __all__ = [
     "lines_of",
     "interesting_lines",
     "show_param_cov_matrix",
+    "pmtot",
     "dmxparse",
     "dmxstats",
     "dmx_ranges_old",
     "dmx_ranges",
-    "p_to_f",
-    "pferrs",
     "weighted_mean",
     "ELL1_check",
-    "mass_funct",
-    "mass_funct2",
-    "pulsar_mass",
-    "companion_mass",
-    "pulsar_age",
-    "pulsar_edot",
-    "pulsar_B",
-    "pulsar_B_lightcyl",
     "FTest",
     "add_dummy_distance",
     "remove_dummy_distance",
+    "info_string",
 ]
 
 
 # Actual exported tools
 
 
-class PosVel(object):
+class PosVel:
     """Position/Velocity class.
 
     The class is used to represent the 6 values describing position
@@ -308,17 +335,22 @@ def taylor_horner(x, coeffs):
         In [1]: taylor_horner(2.0, [10, 3, 4, 12])
         Out[1]: 40.0
 
+    Parameters
+    ----------
+    x: astropy.units.Quantity
+        Input value; may be an array.
+    coeffs: list of astropy.units.Quantity
+        Coefficient array; must have length at least one. The coefficient in
+        position ``i`` is multiplied by ``x**i``. Each coefficient should
+        just be a number, not an array. The units should be compatible once
+        multiplied by an appropriate power of x.
+
+    Returns
+    -------
+    astropy.units.Quantity
+        Output value; same shape as input. Units as inferred from inputs.
     """
-    result = 0.0
-    if hasattr(coeffs[-1], "unit"):
-        if not hasattr(x, "unit"):
-            x = x * u.Unit("")
-        result *= coeffs[-1].unit / x.unit
-    fact = float(len(coeffs))
-    for coeff in coeffs[::-1]:
-        result = result * x / fact + coeff
-        fact -= 1.0
-    return result
+    return taylor_horner_deriv(x, coeffs, deriv_order=0)
 
 
 def taylor_horner_deriv(x, coeffs, deriv_order=1):
@@ -330,6 +362,23 @@ def taylor_horner_deriv(x, coeffs, deriv_order=1):
         In [1]: taylor_horner_deriv(2.0, [10, 3, 4, 12], 1)
         Out[1]: 15.0
 
+    Parameters
+    ----------
+    x: astropy.units.Quantity
+        Input value; may be an array.
+    coeffs: list of astropy.units.Quantity
+        Coefficient array; must have length at least one. The coefficient in
+        position ``i`` is multiplied by ``x**i``. Each coefficient should
+        just be a number, not an array. The units should be compatible once
+        multiplied by an appropriate power of x.
+    deriv_order: int
+        The order of the derivative to take (that is, how many times to differentiate).
+        Must be non-negative.
+
+    Returns
+    -------
+    astropy.units.Quantity
+        Output value; same shape as input. Units as inferred from inputs.
     """
     result = 0.0
     if hasattr(coeffs[-1], "unit"):
@@ -337,7 +386,7 @@ def taylor_horner_deriv(x, coeffs, deriv_order=1):
             x = x * u.Unit("")
         result *= coeffs[-1].unit / x.unit
     der_coeffs = coeffs[deriv_order::]
-    fact = float(len(der_coeffs))
+    fact = len(der_coeffs)
     for coeff in der_coeffs[::-1]:
         result = result * x / fact + coeff
         fact -= 1.0
@@ -353,7 +402,7 @@ def open_or_use(f, mode="r"):
     a subclass of ``str`` will be passed through untouched.
 
     """
-    if isinstance(f, six.string_types):
+    if isinstance(f, (str, bytes)):
         with open(f, mode) as fl:
             yield fl
     else:
@@ -384,7 +433,7 @@ def interesting_lines(lines, comments=None):
     """
     if comments is None:
         cc = ()
-    elif isinstance(comments, six.string_types):
+    elif isinstance(comments, (str, bytes)):
         cc = (comments,)
     else:
         cc = tuple(comments)
@@ -412,7 +461,18 @@ def show_param_cov_matrix(matrix, params, name="Covariance Matrix", switchRD=Fal
     :param name: title to be printed above, default Covariance Matrix
     :param switchRD: if True, switch the positions of RA and DEC to match setup of TEMPO cov. matrices
 
-    :return string to be printed"""
+    :return: string to be printed
+
+    Warning
+    -------
+    **DEPRECATED**.  Use :meth:`pint.pint_matrix.CovarianceMatrix.prettyprint` instead of :func:`show_param_cov_matrix`
+    """
+
+    warn(
+        "This method is deprecated. Use `parameter_covariance_matrix.prettyprint()` instead of `show_param_cov_matrix()`",
+        category=DeprecationWarning,
+    )
+
     output = StringIO()
     matrix = deepcopy(matrix)
     try:
@@ -474,14 +534,19 @@ def pmtot(model):
     so PMRA = (d(RAJ)/dt)*cos(DECJ). This is different from the astrometry community where mu_alpha = d(alpha)/dt.
     Thus, we don't need to include cos(DECJ) or cos(ELAT) in our calculation.
 
+    Parameters
+    ----------
+    model: pint.models.timing_model.TimingModel
+
     Returns
     -------
-    pmtot : Quantity
-        Returns total proper motion with units of u.mas/u.yr
+    pmtot : astropy.units.Quantity
+        Returns total proper motion with units of ``u.mas/u.yr``
 
     Raises
     ------
-        AttributeError if no Astrometry component is found in the model
+    AttributeError
+        If no Astrometry component is found in the model
     """
 
     if "AstrometryEcliptic" in model.components.keys():
@@ -547,8 +612,8 @@ def dmx_ranges_old(
     component : TimingModel.Component object
         A DMX Component class with the DMX ranges included
     """
-    from pint.models.timing_model import Component
     import pint.models.parameter
+    from pint.models.timing_model import Component
 
     MJDs = toas.get_mjds()
     freqs = toas.table["freq"]
@@ -644,7 +709,7 @@ def dmx_ranges_old(
                 print("Ack!  This shouldn't be happening!")
             oldmax = DMX.max
     # Init mask to all False
-    mask = np.zeros_like(MJDs.value, dtype=np.bool)
+    mask = np.zeros_like(MJDs.value, dtype=bool)
     # Mark TOAs as True if they are in any DMX bin
     for DMX in DMXs:
         mask[np.logical_and(MJDs > DMX.min - offset, MJDs < DMX.max + offset)] = True
@@ -716,8 +781,8 @@ def dmx_ranges(toas, divide_freq=1000.0 * u.MHz, binwidth=15.0 * u.d, verbose=Fa
     component : TimingModel.Component object
         A DMX Component class with the DMX ranges included
     """
-    from pint.models.timing_model import Component
     import pint.models.parameter
+    from pint.models.timing_model import Component
 
     MJDs = toas.get_mjds()
     freqs = toas.table["freq"].quantity
@@ -755,7 +820,7 @@ def dmx_ranges(toas, divide_freq=1000.0 * u.MHz, binwidth=15.0 * u.d, verbose=Fa
             DMX.sum_print()
 
     # Init mask to all False
-    mask = np.zeros_like(MJDs.value, dtype=np.bool)
+    mask = np.zeros_like(MJDs.value, dtype=bool)
     # Mark TOAs as True if they are in any DMX bin
     for DMX in DMXs:
         mask[np.logical_and(MJDs >= DMX.min, MJDs <= DMX.max)] = True
@@ -928,38 +993,20 @@ def dmxparse(fitter, save=False):
         DMX_keys_ma = None
 
     # Make sure that the fitter has a covariance matrix, otherwise return the initial values
-    if hasattr(fitter, "covariance_matrix"):
+    if hasattr(fitter, "parameter_covariance_matrix"):
         # now get the full parameter covariance matrix from pint
-        # NOTE: we will need to increase all indices by 1 to account for the 'Offset' parameter
-        # that is the first index of the designmatrix
-        params = np.array(fitter.model.free_params)
-        p_cov_mat = fitter.covariance_matrix
-        # Now we get the indices that correspond to the DMX values
-        DMX_p_idxs = np.zeros(len(dmx_epochs), dtype=int)
-        for ii in range(len(dmx_epochs)):
-            if DMX_keys_ma is None:
-                key = DMX_keys[ii]
-            else:
-                key = DMX_keys_ma[ii]
-            if "DMX" not in key:
-                pass
-            else:
-                DMX_p_idxs[ii] = (
-                    int(np.where(params == key)[0]) + 1
-                )  # extra 1 is for offset parameters
-        # Sort the array in numerical order for 2.7. 3.5
-        DMX_p_idxs = np.trim_zeros(np.sort(DMX_p_idxs))
-        # Define a matrix that is just the DMX covariances
-        cc = p_cov_mat[
-            DMX_p_idxs[0] : DMX_p_idxs[-1] + 1, DMX_p_idxs[0] : DMX_p_idxs[-1] + 1
-        ]
+        # access by label name to make sure we get the right values
+        # make sure they are sorted in ascending order
+        cc = fitter.parameter_covariance_matrix.get_label_matrix(
+            sorted(["DMX_" + x for x in dmx_epochs])
+        )
         n = len(DMX_Errs) - np.sum(mask_idxs)
         # Find error in mean DM
         DMX_mean = np.mean(DMXs)
-        DMX_mean_err = np.sqrt(cc.sum()) / float(n)
+        DMX_mean_err = np.sqrt(cc.matrix.sum()) / float(n)
         # Do the correction for varying DM
         m = np.identity(n) - np.ones((n, n)) / float(n)
-        cc = np.dot(np.dot(m, cc), m)
+        cc = np.dot(np.dot(m, cc.matrix), m)
         DMX_vErrs = np.zeros(n)
         # We also need to correct for the units here
         for i in range(n):
@@ -1062,7 +1109,7 @@ def weighted_mean(arrin, weights_in, inputmean=None, calcerr=False, sdev=False):
     Converted from IDL: 2006-10-23. Erin Sheldon, NYU
     Copied from PRESTO to PINT : 2020-04-18
 
-   """
+    """
     arr = arrin
     weights = weights_in
     wtot = weights.sum()
@@ -1086,207 +1133,24 @@ def weighted_mean(arrin, weights_in, inputmean=None, calcerr=False, sdev=False):
         return wmean, werr
 
 
-def p_to_f(p, pd, pdd=None):
-    """Converts P, Pdot to F, Fdot (or vice versa)
-
-    Convert period, period derivative and period second
-    derivative to the equivalent frequency counterparts.
-    Will also convert from f to p.
-    """
-    f = 1.0 / p
-    fd = -pd / (p * p)
-    if pdd is None:
-        return [f, fd]
-    else:
-        if pdd == 0.0:
-            fdd = 0.0
-        else:
-            fdd = 2.0 * pd * pd / (p ** 3.0) - pdd / (p * p)
-        return [f, fd, fdd]
-
-
-def pferrs(porf, porferr, pdorfd=None, pdorfderr=None):
-    """Convert P, Pdot to F, Fdot with uncertainties.
-
-    Calculate the period or frequency errors and
-    the pdot or fdot errors from the opposite one.
-    """
-    if pdorfd is None:
-        return [1.0 / porf, porferr / porf ** 2.0]
-    else:
-        forperr = porferr / porf ** 2.0
-        fdorpderr = np.sqrt(
-            (4.0 * pdorfd ** 2.0 * porferr ** 2.0) / porf ** 6.0
-            + pdorfderr ** 2.0 / porf ** 4.0
-        )
-        [forp, fdorpd] = p_to_f(porf, pdorfd)
-        return [forp, forperr, fdorpd, fdorpderr]
-
-
-def pulsar_age(f, fdot, n=3, fo=1e99 * u.Hz):
-    """Compute pulsar characteristic age
-
-    Return the age of a pulsar given the spin frequency
-    and frequency derivative.  By default, the characteristic age
-    is returned (assuming a braking index 'n'=3 and an initial
-    spin frequency fo >> f).  But 'n' and 'fo' can be set.
-    """
-    return (-f / ((n - 1.0) * fdot) * (1.0 - (f / fo) ** (n - 1.0))).to(u.yr)
-
-
-def pulsar_edot(f, fdot, I=1.0e45 * u.g * u.cm ** 2):
-    """Compute pulsar spindown energy loss rate
-
-    Return the pulsar Edot (in erg/s) given the spin frequency and
-    frequency derivative. The NS moment of inertia is assumed to be
-    I = 1.0e45 g cm^2 by default.
-    """
-    return (-4.0 * np.pi ** 2 * I * f * fdot).to(u.erg / u.s)
-
-
-def pulsar_B(f, fdot):
-    """Compute pulsar surface magnetic field
-
-    Return the estimated pulsar surface magnetic field strength
-    given the spin frequency and frequency derivative.
-    """
-    # This is a hack to use the traditional formula by stripping the units.
-    # It would be nice to improve this to a  proper formula with units
-    return 3.2e19 * u.G * np.sqrt(-fdot.to(u.Hz / u.s).value / f.to(u.Hz).value ** 3.0)
-
-
-def pulsar_B_lightcyl(f, fdot):
-    """Compute pulsar magnetic field at the light cylinder
-
-    Return the estimated pulsar magnetic field strength at the
-    light cylinder given the spin frequency and
-    frequency derivative.
-    """
-    p, pd = p_to_f(f, fdot)
-    # This is a hack to use the traditional formula by stripping the units.
-    # It would be nice to improve this to a  proper formula with units
-    return (
-        2.9e8
-        * u.G
-        * p.to(u.s).value ** (-5.0 / 2.0)
-        * np.sqrt(pd.to(u.dimensionless_unscaled).value)
-    )
-
-
-def mass_funct(pb, x):
-    """Compute binary mass function from period and semi-major axis
-
-    Parameters
-    ----------
-    pb : Quantity
-        Binary period
-    x : Quantity in `pint.ls`
-        Semi-major axis, A1SINI, in units of ls
-
-    Returns
-    -------
-    f_m : Quantity
-        Mass function in solar masses
-    """
-    fm = 4.0 * np.pi ** 2 * x ** 3 / (const.G * pb ** 2)
-    return fm.to(u.solMass)
-
-
-def mass_funct2(mp, mc, i):
-    """Compute binary mass function from masses and inclination
-
-    Parameters
-    ----------
-    mp : Quantity
-        Pulsar mass, typically in u.solMass
-    mc : Quantity
-        Companion mass, typically in u.solMass
-    i : Angle
-        Inclination angle, in u.deg or u.rad
-
-    Notes
-    -----
-    Inclination is such that edge on is `i = 90*u.deg`
-    An 'average' orbit has cos(i) = 0.5, or `i = 60*u.deg`
-    """
-    return (mc * np.sin(i)) ** 3.0 / (mc + mp) ** 2.0
-
-
-def pulsar_mass(pb, x, mc, inc):
-    """Compute pulsar mass from orbit and Shapiro delay parameters
-
-    Return the pulsar mass (in solar mass units) for a binary.
-    Finds the value using a bisection technique.
-
-    Parameters
-    ----------
-    pb : Quantity
-        Binary orbital period
-    x : Quantity
-        Projected semi-major axis (aka ASINI) in `pint.ls`
-    mc : Quantity
-        Companion mass in u.solMass
-    inc : Angle
-        Inclination angle, in u.deg or u.rad
-
-    Returns
-    -------
-    mass : Quantity in u.solMass
-    """
-    massfunct = mass_funct(pb, x)
-
-    # Do some unit manipulation here so that scipy bisect doesn't see the units
-    def localmf(mp, mc=mc, mf=massfunct, i=inc):
-        return (mass_funct2(mp * u.solMass, mc, i) - mf).value
-
-    return zeros.bisect(localmf, 0.0, 1000.0) * u.solMass
-
-
-def companion_mass(pb, x, inc=60.0 * u.deg, mpsr=1.4 * u.solMass):
-    """Commpute the companion mass from the orbital parameters
-
-    Compute companion mass for a binary system from orbital mechanics,
-    not Shapiro delay.
-
-    Parameters
-    ----------
-    pb : Quantity
-        Binary orbital period
-    x : Quantity
-        Projected semi-major axis (aka ASINI) in `pint.ls`
-    inc : Angle, optional
-        Inclination angle, in u.deg or u.rad. Default is 60 deg.
-    mpsr : Quantity, optional
-        Pulsar mass in u.solMass. Default is 1.4 Msun
-
-    Returns
-    -------
-    mass : Quantity in u.solMass
-    """
-    massfunct = mass_funct(pb, x)
-
-    # Do some unit manipulation here so that scipy bisect doesn't see the units
-    def localmf(mc, mp=mpsr, mf=massfunct, i=inc):
-        return (mass_funct2(mp, mc * u.solMass, i) - mf).value
-
-    return zeros.bisect(localmf, 0.001, 1000.1) * u.solMass
-
-
-def ELL1_check(A1, E, TRES, NTOA, outstring=True):
+@u.quantity_input
+def ELL1_check(
+    A1: u.cm, E: u.dimensionless_unscaled, TRES: u.us, NTOA: int, outstring=True
+):
     """Check for validity of assumptions in ELL1 binary model
 
     Checks whether the assumptions that allow ELL1 to be safely used are
     satisfied. To work properly, we should have:
-    asini/c * ecc**2 << timing precision / sqrt(# TOAs)
-    or A1 * E**2 << TRES / sqrt(NTOA)
+    :math:`asini/c  e^2 \ll {\\rm timing precision} / \sqrt N_{\\rm TOA}`
+    or :math:`A1 E^2 \ll TRES / \sqrt N_{\\rm TOA}`
 
     Parameters
     ----------
-    A1 : Quantity
+    A1 : astropy.units.Quantity
         Projected semi-major axis (aka ASINI) in `pint.ls`
-    E : Quantity (dimensionless)
+    E : astropy.units.Quantity (dimensionless)
         Eccentricity
-    TRES : Quantity
+    TRES : astropy.units.Quantity
         RMS TOA uncertainty
     NTOA : int
         Number of TOAs in the fit
@@ -1323,33 +1187,8 @@ def ELL1_check(A1, E, TRES, NTOA, outstring=True):
         return False
 
 
-def shklovskii_factor(pmtot, D):
-    """
-    Return magnitude of Shklovskii correction factor
-
-    Computes the Shklovskii correction factor, as defined in Eq 8.12 of Lorimer & Kramer (2005)
-    This is the factor by which Pdot/P is increased due to the transverse velocity.
-    Note that this affects both the measured spin period and the orbital period.
-    If we call this Shklovskii acceleration a_s, then
-        Pdot_intrinsic = Pdot_observed - a_s*P
-
-    Parameters
-    ----------
-    pmtot : Quantity, typically units of u.mas/u.yr
-        Total proper motion of the pulsar (system)
-    D : Quantity, typically in units of u.kpc or u.pc
-        Distance to the pulsar
-    """
-    # This uses the small angle approximation that sin(x) = x, so we need to
-    # make our angle dimensionless.
-    with u.set_enabled_equivalencies(u.dimensionless_angles()):
-        a_s = (D * pmtot ** 2 / const.c).to(u.s ** -1)
-    return a_s
-
-
 def FTest(chi2_1, dof_1, chi2_2, dof_2):
-    """
-    Run F-test.
+    """Run F-test.
 
     Compute an F-test to see if a model with extra parameters is
     significant compared to a simpler model.  The input values are the
@@ -1365,18 +1204,18 @@ def FTest(chi2_1, dof_1, chi2_2, dof_2):
 
     Parameters
     -----------
-    chi2_1 : Float
+    chi2_1 : float
         Chi-squared value of model with fewer parameters
-    dof_1 : Int
+    dof_1 : int
         Degrees of freedom of model with fewer parameters
-    chi2_2 : Float
+    chi2_2 : float
         Chi-squared value of model with more parameters
-    dof_2 : Int
+    dof_2 : int
         Degrees of freedom of model with more parameters
 
     Returns
     --------
-    ft : Float
+    ft : float
         F-test significance value for the model with the larger number of
         components over the other.
     """
@@ -1384,35 +1223,36 @@ def FTest(chi2_1, dof_1, chi2_2, dof_2):
     if delta_chi2 > 0 and dof_1 != dof_2:
         delta_dof = dof_1 - dof_2
         new_redchi2 = chi2_2 / dof_2
-        F = np.float64(
-            (delta_chi2 / delta_dof) / new_redchi2
-        )  # fdtr doesn't like float128
+        F = float((delta_chi2 / delta_dof) / new_redchi2)  # fdtr doesn't like float128
         ft = fdtrc(delta_dof, dof_2, F)
+    elif dof_1 == dof_2:
+        log.warning("Models have equal degrees of freedom, cannot perform F-test.")
+        ft = np.nan
+    elif delta_chi2 <= 0:
+        log.warning(
+            "Chi^2 for Model 2 is larger than Chi^2 for Model 1, cannot perform F-test."
+        )
+        ft = 1.0
     else:
-        if delta_chi2 <= 0:
-            log.warning(
-                "Chi^2 for Model 2 is larger than Chi^2 for Model 1, cannot preform F-test."
-            )
-        elif dof_1 == dof_2:
-            log.warning("Models have equal degrees of freedom, cannot preform F-test.")
-        ft = False
+        raise ValueError(
+            f"Mystery problem in Ftest - maybe NaN? {chi2_1} {dof_1} {chi2_2} {dof_2}"
+        )
     return ft
 
 
 def add_dummy_distance(c, distance=1 * u.kpc):
-    """
-    Adds a dummy distance to a SkyCoord object for applying proper motion
+    """Adds a dummy distance to a SkyCoord object for applying proper motion
 
     Parameters
     ----------
-    c: `astropy.coordinates.sky_coordinate.SkyCoord` object
+    c: astropy.coordinates.SkyCoord
         current SkyCoord object without distance but with proper motion and obstime
-    distance: `Quantity`, optional
+    distance: astropy.units.Quantity, optional
         distance to supply
 
     Returns
     -------
-    cnew
+    cnew : astropy.coordinates.SkyCoord
         new SkyCoord object with a distance attached
     """
 
@@ -1482,17 +1322,16 @@ def add_dummy_distance(c, distance=1 * u.kpc):
 
 
 def remove_dummy_distance(c):
-    """
-    Removes a dummy distance from a SkyCoord object after applying proper motion
+    """Removes a dummy distance from a SkyCoord object after applying proper motion
 
     Parameters
     ----------
-    c: `astropy.coordinates.sky_coordinate.SkyCoord` object
+    c: astropy.coordinates.SkyCoord
         current SkyCoord object with distance and with proper motion and obstime
 
     Returns
     -------
-    cnew
+    cnew : astropy.coordinates.SkyCoord
         new SkyCoord object with a distance removed
     """
 
@@ -1555,3 +1394,216 @@ def remove_dummy_distance(c):
             "Do not know coordinate frame for %r: returning coordinates unchanged" % c
         )
         return c
+
+
+def info_string(prefix_string="# ", comment=None):
+    """Returns an informative string about the current state of PINT.
+
+    Adds:
+
+    * Creation date
+    * PINT version
+    * Username (given by the `gitpython`_ global configuration ``user.name``
+      if available, in addition to :func:`getpass.getuser`).
+    * Host (given by :func:`platform.node`)
+    * OS (given by :func:`platform.platform`)
+    * plus a user-supplied comment (if present).
+
+    Parameters
+    ----------
+    prefix_string: str, default='# '
+        a string to be prefixed to the output (often to designate as a
+        comment or similar)
+    comment: str, optional
+        a free-form comment string to be included if present
+
+    Returns
+    -------
+    str
+        informative string
+
+    Examples
+    --------
+    >>> import pint.utils
+    >>> print(pint.utils.info_string(prefix_string="# ",comment="Example comment"))
+    # Created: 2021-07-21T09:39:45.606894
+    # PINT_version: 0.8.2+311.ge351099d
+    # User: David Kaplan (dlk)
+    # Host: margle-2.local
+    # OS: macOS-10.14.6-x86_64-i386-64bit
+    # Comment: Example comment
+
+    Multi-line comments are allowed:
+
+    >>> import pint.utils
+    >>> print(pint.utils.info_string(prefix_string="C ",
+    ...                              comment="Example multi-line comment\\nAlso using a different comment character"))
+    C Created: 2021-07-21T09:40:34.172333
+    C PINT_version: 0.8.2+311.ge351099d
+    C User: David Kaplan (dlk)
+    C Host: margle-2.local
+    C OS: macOS-10.14.6-x86_64-i386-64bit
+    C Comment: Example multi-line comment
+    C Comment: Also using a different comment character
+
+    Full example of writing a par and tim file:
+
+    >>> from pint.models import get_model_and_toas
+    >>> # the locations of these may vary
+    >>> timfile = "tests/datafile/NGC6440E.tim"
+    >>> parfile = "tests/datafile/NGC6440E.par"
+    >>> m, t = get_model_and_toas(parfile, timfile)
+    >>> print(m.as_parfile(comment="Here is a comment on the par file"))
+    # Created: 2021-07-22T08:24:27.101479
+    # PINT_version: 0.8.2+439.ge81c9b11.dirty
+    # User: David Kaplan (dlk)
+    # Host: margle-2.local
+    # OS: macOS-10.14.6-x86_64-i386-64bit
+    # Comment: Here is a comment on the par file
+    PSR                            1748-2021E
+    EPHEM                               DE421
+    CLK                             UTC(NIST)
+    ...
+
+    >>> from pint.models import get_model_and_toas
+    >>> import io
+    >>> # the locations of these may vary
+    >>> timfile = "tests/datafile/NGC6440E.tim"
+    >>> parfile = "tests/datafile/NGC6440E.par"
+    >>> m, t = get_model_and_toas(parfile, timfile)
+    >>> f = io.StringIO(parfile)
+    >>> t.write_TOA_file(f, comment="Here is a comment on the tim file")
+    >>> f.seek(0)
+    >>> print(f.getvalue())
+    FORMAT 1
+    C Created: 2021-07-22T08:24:27.213529
+    C PINT_version: 0.8.2+439.ge81c9b11.dirty
+    C User: David Kaplan (dlk)
+    C Host: margle-2.local
+    C OS: macOS-10.14.6-x86_64-i386-64bit
+    C Comment: Here is a comment on the tim file
+    unk 1949.609000 53478.2858714192189005 21.710 gbt  -format Princeton -ddm 0.0
+    unk 1949.609000 53483.2767051885165973 21.950 gbt  -format Princeton -ddm 0.0
+    unk 1949.609000 53489.4683897879295023 29.950 gbt  -format Princeton -ddm 0.0
+    ....
+
+
+    Notes
+    -----
+    This can be called via  :func:`~pint.toa.TOAs.write_TOA_file` on a :class:`~~pint.toa.TOAs` object,
+    or :func:`~pint.models.timing_model.TimingModel.as_parfile` on a
+    :class:`~pint.models.timing_model.TimingModel` object.
+
+    .. _gitpython: https://gitpython.readthedocs.io/en/stable/
+    """
+    # try to get the git user if defined
+    try:
+        import git
+
+        # user-level git config
+        c = git.GitConfigParser()
+        username = c.get_value("user", option="name") + f" ({getpass.getuser()})"
+    except (configparser.NoOptionError, configparser.NoSectionError, ImportError):
+        username = getpass.getuser()
+
+    s = f"""
+    Created: {datetime.datetime.now().isoformat()}
+    PINT_version: {pint.__version__}
+    User: {username}
+    Host: {platform.node()}
+    OS: {platform.platform()}
+    """
+
+    s = textwrap.dedent(s)
+    # remove blank lines
+    s = os.linesep.join([x for x in s.splitlines() if x])
+    if comment is not None:
+        if os.linesep in comment:
+            s += os.linesep + os.linesep.join(
+                [f"Comment: {x}" for x in comment.splitlines()]
+            )
+        else:
+            s += f"{os.linesep}Comment: {comment}"
+
+    if (prefix_string is not None) and (len(prefix_string) > 0):
+        s = os.linesep.join([prefix_string + x for x in s.splitlines()])
+    return s
+
+
+def list_parameters(class_=None):
+    """List parameters understood by PINT.
+
+    Parameters
+    ----------
+    class_: type, optional
+        If provided, produce a list of parameters understood by the Component type; if None,
+        return a list of parameters understood by all Components known to PINT.
+
+    Returns
+    -------
+    list of dict
+        Each entry is a dictionary describing one parameter. Dictionary values are all strings
+        or lists of strings, and will include at least "name", "classes", and "description".
+    """
+    if class_ is not None:
+        from pint.models.parameter import (
+            boolParameter,
+            intParameter,
+            maskParameter,
+            prefixParameter,
+            strParameter,
+        )
+
+        result = []
+        inst = class_()
+        for p in inst.params:
+            pm = getattr(inst, p)
+            d = dict(
+                name=pm.name,
+                class_=f"{class_.__module__}.{class_.__name__}",
+                description=pm.description,
+            )
+            if pm.aliases:
+                d["aliases"] = [a for a in pm.aliases if a != pm.name]
+            if pm.units:
+                d["kind"] = pm.units.to_string()
+                if not d["kind"]:
+                    d["kind"] = "number"
+            elif isinstance(pm, boolParameter):
+                d["kind"] = "boolean"
+            elif isinstance(pm, strParameter):
+                d["kind"] = "string"
+            elif isinstance(pm, intParameter):
+                d["kind"] = "integer"
+            if isinstance(pm, prefixParameter):
+                d["name"] = pm.prefix + "{number}"
+                d["aliases"] = [a + "{number}" for a in pm.prefix_aliases]
+            if isinstance(pm, maskParameter):
+                d["name"] = pm.origin_name + " {flag} {value}"
+                d["aliases"] = [a + " {flag} {value}" for a in pm.prefix_aliases]
+            if "aliases" in d and not d["aliases"]:
+                del d["aliases"]
+            result.append(d)
+        return result
+    else:
+        import pint.models.timing_model
+
+        results = {}
+        ct = pint.models.timing_model.Component.component_types.copy()
+        ct["TimingModel"] = pint.models.timing_model.TimingModel
+        for v in ct.values():
+            for d in list_parameters(v):
+                n = d["name"]
+                class_ = d.pop("class_")
+                if n not in results:
+                    d["classes"] = [class_]
+                    results[n] = d
+                else:
+                    r = results[n].copy()
+                    r.pop("classes")
+                    if r != d:
+                        raise ValueError(
+                            f"Parameter {d} in class {class_} does not match {results[n]}"
+                        )
+                    results[n]["classes"].append(class_)
+        return sorted(results.values(), key=lambda d: d["name"])
